module Kristall_Modul

! Allgemeine Parameter
integer,parameter::DIM=2                 ! Raumdimension (2D oder 3D)
integer,parameter::AUSGEBEN_ALLE=20
real,parameter::   DELTA_T=0.01           ! Zeitschrittweite 
real,parameter::   T_MAX=50.0
integer,parameter::NX=25             
integer,parameter::NY=10
! Physikalische Konstanten
real,parameter::SIGMA=1.0           
real,parameter::EPS=5.0           
real,parameter::MASSE=1.0          
real,parameter::TEMP=1.5           

! Eigene Datentypen
type Atom                                ! Himmelskoerper:
  real::m                                !   Masse
  real,dimension(DIM)::x,v,F,F_alt             !   Ort,Geschwindigkeit,Kraft
  integer::typ                           !   Typ (fuer Darstellung)
end type
type Atome                               ! Sonnensystem:
  integer::N                             !   Anzahl der Himmelskoerper
  type(Atom),dimension(:),pointer::a     !   Feld der Himmelskoerper
end type

contains

! ***  Zeitintegration und Ausgabe  *******************************************

subroutine Ausgabe(A,t)
      type(Atome)::A
      real::t
  integer::i
  write(unit=1,fmt=*) A%N
  write(unit=1,fmt=*) t," fs"
  do i=1,A%N
    write(unit=1,fmt=*) A%a(i)%typ, A%a(i)%x, 0, 0
  end do
end subroutine 

subroutine Kraft(A,i,j)
      type(Atome)::A; integer::i,j
  real,dimension(DIM)::xij,Fij
  real::rr
  integer::d
  real::sigma_r_6
  xij=A%a(j)%x-A%a(i)%x                  ! Abstandsvektor von i nach j
  rr=sum(xij**2)
  sigma_r_6=SIGMA**6/(rr**3)
  
  Fij=24*EPS/rr* sigma_r_6  *(1-2*(sigma_r_6)) *xij     ! Kraft von j auf i
  A%a(i)%F=A%a(i)%F+Fij
  A%a(j)%F=A%a(j)%F-Fij
end subroutine

subroutine Berechne_F(A)
      type(Atome)::A
  integer::i
  integer::j
  do i=1,A%N
    A%a(i)%F_alt = A%a(i)%F
    A%a(i)%F = 0
  end do

  do i=1,A%N
    do j=i+1,A%N
      call Kraft(A,i,j)
    end do
  end do
end subroutine

subroutine Berechne_x(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%x=A%a(i)%x+dt*A%a(i)%v + dt**2*A%a(i)%F/(2*A%a(i)%m)
  end do
end subroutine

subroutine Berechne_v(A,dt)
      type(Atome)::A; real::dt
  integer::i  
  do i=1,A%N
    A%a(i)%v=A%a(i)%v+dt*(A%a(i)%F + A%a(i)%F_alt)/(2*A%a(i)%m) 
  end do
end subroutine

subroutine Zeitintegration(A,dt,t_end)
      type(Atome)::A; real::dt,t_end
  integer::k
  real::t
  t=0
  k=0                                    ! Zaehler fuer Ausgabe
  call Berechne_F(A)
  do while(t<t_end)
    t=t+dt
    call Berechne_x(A,dt)
    call Berechne_F(A)
    call Berechne_v(A,dt) 
    k=k+1
    if(mod(k,AUSGEBEN_ALLE)==0) call Ausgabe(A,t)
  end do
end subroutine

! ***  Startkonfiguration  ****************************************************

function NV_Zufallszahl()
      real::NV_Zufallszahl
  real::a1,a2,r
  r=0
  do while ((r==0).or.(1<=r))
    call random_number(a1); a1=2*a1-1
    call random_number(a2); a2=2*a2-1
    r=a1**2+a2**2
  end do
  NV_Zufallszahl=a1*sqrt(-2*log(r)/r)
end function

subroutine Temperatur(A, T)
      type(Atome)::A
      real::T
  integer::i,d
  real::s,z
  do i=1,A%N
    s=sqrt(T/A%a(i)%m)
    do d=1,DIM
      z=NV_Zufallszahl()*s
      A%a(i)%v(d)=A%a(i)%v(d)+z
    end do
  end do
end subroutine

subroutine Startkonfiguration(A)
      type(Atome)::A
  integer::i,j
  real,parameter::RGG=2**(1.0/6.0)*SIGMA
  ! Partikel initialisieren
  A%N=NX*NY
  allocate(A%a(A%N))
  do i=1,NX
    do j=1,NY
      A%a((i-1)*NY+j)%typ=1; A%a((i-1)*NY+j)%m=MASSE; A%a((i-1)*NY+j)%x=(/(i-1)*RGG,j*RGG/); A%a((i-1)*NY+j)%v=(/0.0,0.0/);
    end do 
  end do 
  call Temperatur(A,TEMP)
end subroutine

end module

! ***  Hauptprogramm  *********************************************************

program Kristall

use Kristall_Modul

type(Atome)::A

open(unit=1,file="kristall.xyz",action="write",status="replace")
call Startkonfiguration(A)
call Ausgabe(A,0.0)

call Zeitintegration(A,DELTA_T,T_MAX)

close(unit=1)
deallocate(A%a)

end program 
