module Planeten_Modul

! Allgemeine Parameter
integer,parameter::DIM=3                 ! Raumdimension (2D oder 3D)
integer,parameter::AUSGEBEN_ALLE=1000
real,parameter::   DELTA_T=0.0001           ! Zeitschrittweite 
real,parameter::   T_MAX=200             ! Endzeit
! Physikalische Konstanten
real,parameter::GAMMA=1.177e-4           ! Gravitationskonstante GAMMA

! Eigene Datentypen
type Atom                                ! Himmelskoerper:
  real::m                                !   Masse
  real,dimension(DIM)::x,v,F             !   Ort,Geschwindigkeit,Kraft
  integer::typ                           !   Typ (fuer Darstellung)
end type
type Atome                               ! Sonnensystem:
  integer::N                             !   Anzahl der Himmelskoerper
  type(Atom),dimension(:),pointer::a     !   Feld der Himmelskoerper
end type

contains

! ***  Zeitintegration und Ausgabe  *******************************************

subroutine Ausgabe(A,t)
      type(Atome)::A
      real::t
  integer::i
  write(unit=1,fmt=*) A%N
  write(unit=1,fmt=*) t," fs"
  do i=1,A%N
    write(unit=1,fmt=*) A%a(i)%typ, A%a(i)%x, 0, 0
  end do
end subroutine 

subroutine Kraft(A,i,j)
      type(Atome)::A; integer::i,j
  real,dimension(DIM)::xij
  real::r,rr,Fij
  integer::d
  xij=A%a(j)%x-A%a(i)%x                  ! Abstandsvektor von i nach j
  rr=0.0                                 ! r^2
  do d=1,DIM
    rr=rr+xij(d)**2
  end do
  r=sqrt(rr)                             ! Abstand
  Fij=GAMMA*A%a(i)%m*A%a(j)%m/rr         ! Kraft von j auf i
  A%a(i)%F=A%a(i)%F+Fij*xij/r
end subroutine

subroutine Berechne_F(A)
      type(Atome)::A
  integer::i
  integer::j
  do i=1,A%N
    A%a(i)%F = 0
    do j=1,A%N
	if (i /= j) call Kraft(A,i,j)
    end do
  end do
end subroutine

subroutine Berechne_x(A,dt)
      type(Atome)::A; real::dt
  integer::i
  do i=1,A%N
    A%a(i)%x=A%a(i)%x+dt*A%a(i)%v
  end do
end subroutine

subroutine Berechne_v(A,dt)
      type(Atome)::A; real::dt
  integer::i  
  do i=1,A%N
    A%a(i)%v=A%a(i)%v+dt*A%a(i)%F/A%a(i)%m
  end do
end subroutine

subroutine Zeitintegration(A,dt,t_end)
      type(Atome)::A; real::dt,t_end
  integer::k
  real::t
  t=0
  k=0                                    ! Zaehler fuer Ausgabe
  do while(t<t_end)
    t=t+dt
    call Berechne_F(A)
    call Berechne_x(A,dt)
    call Berechne_v(A,dt) 
    k=k+1
    if(mod(k,AUSGEBEN_ALLE)==0) call Ausgabe(A,t)
  end do
end subroutine

! ***  Startkonfiguration  ****************************************************

subroutine Startkonfiguration(A)
      type(Atome)::A
  ! Partikel initialisieren
  A%N=4
  allocate(A%a(A%N))
  ! Sonne:
  A%a(1)%typ=1; A%a(1)%m=333e3; A%a(1)%x=(/ 0.0,0.0,0.0/); A%a(1)%v=(/0.0,-2.75e-3,0.0/);
  ! Erde:
  A%a(2)%typ=2; A%a(2)%m=  1.0; A%a(2)%x=(/ 1.0,0.0,0.0/); A%a(2)%v=(/0.0,6.28,0.0/);
  ! Jupiter:
  A%a(3)%typ=3; A%a(3)%m=  318.0; A%a(3)%x=(/ 5.4,0.0,0.0/); A%a(3)%v=(/0.0,2.86,0.0/);
  ! Komet Halley:
  A%a(4)%typ=4; A%a(4)%m= 3e-11; A%a(4)%x=(/ 35.1,0.0,0.0/); A%a(4)%v=(/0.0,0.192, 1.0/);
 
end subroutine

end module

! ***  Hauptprogramm  *********************************************************

program Planeten

use Planeten_Modul

type(Atome)::A

open(unit=1,file="planeten.xyz",action="write",status="replace")
call Startkonfiguration(A)
call Ausgabe(A,0.0)

call Zeitintegration(A,DELTA_T,T_MAX)

close(unit=1)
deallocate(A%a)

end program 
